package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.ChannelDTO;
import com.zanxiang.manage.domain.dto.GameDTO;
import com.zanxiang.manage.domain.dto.UserDTO;
import com.zanxiang.manage.domain.dto.UserExtDTO;
import com.zanxiang.manage.domain.vo.GameUserVO;
import com.zanxiang.manage.domain.vo.UserVO;
import com.zanxiang.manage.service.*;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.mybatis.mapper.UserMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : 用户玩家
 */
@Slf4j
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {

    @Autowired
    private UserExtService userExtService;

    @Autowired
    private GameService gameService;

    @Autowired
    private ChannelService channelService;

    @Autowired
    private GameUserService gameUserService;

    /**
     * 查询玩家详细信息
     *
     * @param userId : 用户id
     * @return : 返回用户信息
     */
    @Override
    public UserVO getUserInfo(Long userId) {
        //查询用户信息
        User user = super.getById(userId);
        if (user == null) {
            throw new BaseException("用户信息不存在!");
        }
        //查询用户拓展信息
        UserExtDTO userExtDTO = userExtService.getById(userId);
        if (userExtDTO == null) {
            throw new BaseException("用户拓展信息不存在!");
        }
        UserVO userVO = BeanUtils.copy(user, UserVO.class);
        //拓展信息补充
        userVO.setRealName(userExtDTO.getRealName());
        userVO.setBirthday(userExtDTO.getBirthday());
        userVO.setSex(userExtDTO.getSex());
        userVO.setIdCard(userExtDTO.getIdCard());
        userVO.setRegMobile(userExtDTO.getRegMobile());
        userVO.setRegEmail(userExtDTO.getRegEmail());
        userVO.setRegFrom(userExtDTO.getRegFrom());
        userVO.setRegIp(userExtDTO.getRegIp());
        //注册游戏
        GameDTO gameDTO = gameService.getById(userExtDTO.getRegGameId());
        if (gameDTO != null) {
            userVO.setRegGame(gameDTO.getName());
        }
        //渠道
        ChannelDTO channelDTO = channelService.getById(user.getAgentId());
        if (channelDTO != null) {
            userVO.setChannelName(channelDTO.getChannelName());
        }
        //vip等级
        GameUserVO maxVipGameUser = gameUserService.getMaxVipGameUser(userId);
        if (maxVipGameUser != null){
            userVO.setVipMax(maxVipGameUser.getRoleVipMax());
        }
        //地址
        userVO.setAddress("贝达梦工厂E座二楼趣程科技");
        return userVO;
    }


    /**
     * 玩家相关条件查询用户信息
     *
     * @param userId    : 用户id
     * @param channelId : 渠道id
     * @param userName  : 用户名称
     * @param nickname  : 用户昵称
     * @return : 返回用户信息map
     */
    @Override
    public Map<Long, UserDTO> userCondition(Long userId, Long channelId, String userName, String nickname) {
        //用户信息条件处理
        if (userId == null && channelId == null && Strings.isBlank(userName) && Strings.isBlank(nickname)) {
            return null;
        }
        //玩家账号和玩家昵称涉及的玩家条件
        List<UserDTO> userList = this.getUserList(userId, channelId, userName, nickname);
        if (CollectionUtils.isEmpty(userList)) {
            return Collections.emptyMap();
        }
        return userList.stream().collect(Collectors.toMap(UserDTO::getId, Function.identity()));
    }

    /**
     * 根据玩家账号或者昵称模糊查询玩家列表
     *
     * @param userId    : 用户id
     * @param channelId : 渠道id
     * @param userName  : 玩家账号
     * @param nickname  : 玩家昵称
     * @return : 返回玩家信息
     */
    @Override
    public List<UserDTO> getUserList(Long userId, Long channelId, String userName, String nickname) {
        if (userId == null && channelId != null && Strings.isBlank(userName) && Strings.isBlank(nickname)) {
            return Collections.emptyList();
        }
        //查询
        List<User> userList = super.list(new LambdaQueryWrapper<User>()
                .eq(userId != null, User::getId, userId)
                .eq(channelId != null, User::getAgentId, channelId)
                .like(Strings.isNotBlank(userName), User::getUsername, userName)
                .like(Strings.isNotBlank(nickname), User::getNickname, nickname)
                .select(User::getId, User::getUsername, User::getNickname));
        if (CollectionUtils.isEmpty(userList)) {
            return Collections.emptyList();
        }
        return BeanUtils.copyList(userList, UserDTO.class);
    }

    /**
     * 根据用户id查询信息
     *
     * @param userId : 返回用户信息
     * @return : 返回用户信息
     */
    @Override
    public UserDTO getById(Long userId) {
        return BeanUtils.copy(super.getById(userId), UserDTO.class);
    }

}
